#
#
# = Define: tp::netinstall
#
# This define installs application from their upstream source code
#
#
# == Parameters
#
define tp::netinstall (

  $ensure                    = present,

  $source_url                = undef,
  $source_format             = undef,

  $destination_dir           = "/opt/${title}",
  $destination_dir_owner     = 'root',
  $destination_dir_group     = 'root',
  $destination_dir_mode      = '0644',

  $prerequisite_packages     = { },

  $preinstall_exec_command    = undef,
  $preinstall_exec_cwdmmand    = undef,
  $preinstall_file_source     = undef,
  $preinstall_file_template   = undef,
  $preinstall_file_path       = undef,

  $postinstall_exec_command    = undef,
  $postinstall_file_source     = undef,
  $postinstall_file_template   = undef,
  $postinstall_file_path       = undef,

  $extracted_dir       = '',
  $retrieve_command    = 'wget',
  $retrieve_args       = '',

  $download_dir              = '/var/tmp',

  $exec_timeout              = '3600',
  $exec_path                 = '/bin:/sbin:/usr/bin:/usr/sbin',
  $exec_env                  = [],
  $exec_cwd                  = '/',


  $settings_hash             = { } ,

  $extra_class               = undef,
  $dependency_class          = undef,
  $monitor_class             = undef,
  $firewall_class            = undef,

  $puppi_enable              = false,

  $test_enable               = false,
  $test_acceptance_template  = undef,

  $data_module               = 'tp',

  ) {

  # Parameters validation
  validate_bool($puppi_enable)
  validate_hash($settings_hash)


  # Settings evaluation
  $tp_settings=tp_lookup($title,'settings',$data_module,'merge')
  $settings=merge($tp_settings,$settings_hash)

  # Dependency class
  if $dependency_class { require $dependency_class }

  if $preextract_command {
    exec { "PreExtract ${source_filename} in ${destination_dir} - ${title}":
      command     => $preextract_command,
      subscribe   => Exec["Retrieve ${url} in ${work_dir} - ${title}"],
      refreshonly => true,
      path        => $path,
      environment => $exec_env,
      timeout     => $timeout,
    }
  }

  exec { "Retrieve ${url} in ${work_dir} - ${title}":
    cwd         => $work_dir,
    command     => "${retrieve_command} ${retrieve_args} ${url}",
    creates     => "${work_dir}/${source_filename}",
    timeout     => $timeout,
    path        => $path,
    environment => $exec_env,
  }

  exec { "Extract ${source_filename} from ${work_dir} - ${title}":
    command     => "mkdir -p ${destination_dir} && cd ${destination_dir} && ${real_extract_command} ${work_dir}/${source_filename} ${extract_command_second_arg}",
    unless      => "ls ${destination_dir}/${real_extracted_dir}",
    creates     => "${destination_dir}/${real_extracted_dir}",
    timeout     => $timeout,
    require     => Exec["Retrieve ${url} in ${work_dir} - ${title}"],
    path        => $path,
    environment => $exec_env,
    notify      => Exec["Chown ${source_filename} in ${destination_dir} - ${title}"],
  }

  exec { "Chown ${source_filename} in ${destination_dir} - ${title}":
    command     => "chown -R ${owner}:${group} ${destination_dir}/${real_extracted_dir}",
    refreshonly => true,
    timeout     => $timeout,
    require     => Exec["Extract ${source_filename} from ${work_dir} - ${title}"],
    path        => $path,
    environment => $exec_env,
  }

  if $postextract_command {
    exec { "PostExtract ${source_filename} in ${destination_dir} - ${title}":
      command     => $postextract_command,
      cwd         => $real_postextract_cwd,
      subscribe   => Exec["Extract ${source_filename} from ${work_dir} - ${title}"],
      refreshonly => true,
      timeout     => $timeout,
      require     => [Exec["Retrieve ${url} in ${work_dir} - ${title}"],Exec["Chown ${source_filename} in ${destination_dir} - ${title}"]],
      path        => $path,
      environment => $exec_env,
    }
  }


  # Commons
  if $puppi_enable == true {
    tp::puppi { $title: }
  }

  if $test_enable == true {
    tp::test { $title:
      acceptance_template => $test_acceptance_template,
    }
  }

  # Extra classes
  if $extra_class { include $extra_class }
  if $monitor_class { include $monitor_class }
  if $firewall_class { include $firewall_class }

}
